//
//  BluetoothHomepageViewController.swift
//  MCHPBluetooth
//
//  Created by Ricky Johnson on 10/29/18.
//  Copyright © 2018 Ricky Johnson. All rights reserved.
//

import UIKit
import CoreBluetooth


final class BluetoothHomepageViewController: UIViewController, UITextFieldDelegate, BluetoothClassDelegate{
   

//MARK: IBOutlets
    
    @IBOutlet weak var mainText: UITextView!
    
    @IBOutlet weak var navBar: UINavigationItem!
   
    @IBOutlet weak var userEnteredText: UITextField!
    @IBOutlet weak var textEntryBottomConstraint: NSLayoutConstraint!
    
    var receivedStrings = ""


//MARK: Functions
    
    override func viewDidLoad() {
        super.viewDidLoad()
        

        userEnteredText.delegate = self
        bluetoothConnection = BluetoothClass(delegate: self)
    
        mainText.text = receivedStrings
        
    
        DispatchQueue.main.async {
            if bluetoothConnection.initialized  == true {
                if(bluetoothConnection.manager.state  == .poweredOn){
                    self.navBar.title = "Connected"
                }
                else{
                    self.navBar.title = "Disconnected"
                }
            }
        }
        
       
        /*this creates a notification when a string comes in*/
        NotificationCenter.default.addObserver(self, selector: #selector(incomingString(notification:)), name: .stringReceived, object: nil)
        
        //in order to know when to show and when to hide keyboard
        NotificationCenter.default.addObserver(self, selector: #selector(BluetoothHomepageViewController.keyboardWillShow(_:)), name:NSNotification.Name.UIKeyboardWillShow, object: nil);
        NotificationCenter.default.addObserver(self, selector: #selector(BluetoothHomepageViewController.keyboardWillHide(_:)), name:NSNotification.Name.UIKeyboardWillHide, object: nil);
        
    }
    
    // move the view upwards as keyboard appears
    func keyboardWillShow(_ sender: Notification) {
        if let keyboardFrame: NSValue = sender.userInfo?[UIKeyboardFrameEndUserInfoKey] as? NSValue {
            let keyboardAppearTime = sender.userInfo?[UIKeyboardAnimationDurationUserInfoKey] as? TimeInterval
            let keyboardRectangle = keyboardFrame.cgRectValue
            let keyboardHeight = keyboardRectangle.height
            
            UIView.animate(withDuration: keyboardAppearTime!) {
                self.textEntryBottomConstraint.constant = keyboardHeight + 10
            }
            
        }
    }
    
    // move the keyboard back as keyboard disapears
    
    func keyboardWillHide(_ sender: Notification) {
        if let keyboardFrame: NSValue = sender.userInfo?[UIKeyboardFrameEndUserInfoKey] as? NSValue {
            let keyboardAppearTime = sender.userInfo?[UIKeyboardAnimationDurationUserInfoKey] as? TimeInterval
            let keyboardRectangle = keyboardFrame.cgRectValue
            let keyboardHeight = keyboardRectangle.height
            
            UIView.animate(withDuration: keyboardAppearTime!) {
                self.textEntryBottomConstraint.constant = 8
            }
            
        }
    }
    
    // make the keyboard disapear as user touches outside the  text boxes
    
    override func touchesBegan(_ touches: Set<UITouch>, with event: UIEvent?) {
        view.endEditing(true)
    }
    
    /*causes the done button to prompt an alert
     if the user is not connected send an alert
     if the user is connected and message is empty send alert
     if everythign is all good ask if they'd like to send the message*/
    
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        userEnteredText.resignFirstResponder()
        
        // if we arent connected to a peripheral let the user know that you cant send anything yet
        if (!bluetoothConnection.isconnectedToPeripheral) {
            let alert = UIAlertController(title: "You are not connected", message: "Before you can send a message please connect to a peripheral", preferredStyle: .alert)
            alert.addAction(UIAlertAction(title: "Cancel", style: .cancel, handler:nil))
            present(alert, animated: true, completion: nil)
            
            return true
        }
            // make sure sendable text is in the field
        else if (bluetoothConnection.isconnectedToPeripheral && !((userEnteredText.text?.isEmpty)!)){
            let alert = UIAlertController(title: "Would you like to send this message?", message: "", preferredStyle: .alert)
            alert.addAction(UIAlertAction(title: "Yes", style: .default, handler: { action in
                self.sendMessage(message: self.userEnteredText.text!)
                self.userEnteredText.text = ""
                self.userEnteredText.resignFirstResponder()
            }))
            alert.addAction(UIAlertAction(title: "No", style: .cancel, handler: nil))
            present(alert, animated: true, completion: nil)
            return true
        }
        
        else {
            let alert = UIAlertController(title: "Please enter text to send", message: "", preferredStyle: .alert)
            alert.addAction(UIAlertAction(title: "Dismiss", style: .cancel, handler:nil))
            present(alert, animated: true, completion: nil)
            return true
        }
    }
    
    func sendMessage(message:String){
        if let data = message.data(using: String.Encoding.utf8) {
            bluetoothConnection.connectedPeripheral.writeValue(data, for: bluetoothConnection.writeCharacteristic!, type: .withoutResponse)
        }
    }
    
    /*function that gets executed when the received string notification happens*/
    func incomingString(notification: NSNotification){
        receivedStrings = receivedStrings + bluetoothConnection.receivedString!
        DispatchQueue.main.async {
            self.mainText.text = self.receivedStrings
        }
    }
  
  
    
    
//MARK: IBActions

    @IBAction func clearPressed(_ sender: Any) {
        DispatchQueue.main.async {
            self.receivedStrings = ""
            self.mainText.text = ""
            
        }
    }
  
    
    @IBAction func connectPressed(_ sender: Any) {
        if bluetoothConnection.isconnectedToPeripheral == false {
            performSegue(withIdentifier: "ShowScanner", sender: self)
        } else {
            //TODO:
            let alert = UIAlertController(title: "You Are Connected", message: "You are already connected to a device. If you ", preferredStyle: .alert)
            alert.addAction(UIAlertAction(title: "Cancel", style: .cancel, handler:nil))
            
            alert.addAction(UIAlertAction(title: "Disconnect", style: .default, handler: { action in
                
            bluetoothConnection.manager.cancelPeripheralConnection(bluetoothConnection.connectedPeripheral)
                bluetoothConnection.isconnectedToPeripheral = false
                bluetoothConnection.connectedPeripheral = nil
                self.performSegue(withIdentifier: "ShowScanner", sender: Any?)
            }))
            present(alert, animated: true)
        }
        
    }
    
    @IBAction func returnedFromScanner(segue: UIStoryboardSegue){
        
    }
}

extension Notification.Name {
    static let stringReceived = Notification.Name("stringReceived")
    static let foundPeripheral = Notification.Name("foundPeripheral")
}
